@echo off
::
:: batch script for converting raw lidar data
:: into a set of 250x250 m tiles suitable for contour and vegetation
:: processing.
:: Usage: lidar-batch <lidar_directory\*.laz> or <lidar_directory\*.las>

:: Assumes all the lastools binaries and the perl files to be in the "\data\orientering" directory:
set PATH=%PATH%;\data\orientering
set CORES=7

if exist tiles_classified\tile*.laz goto tiles_generated

:: create buffered tiling

if exist adaptive\tile*.laz goto skip_tile
md adaptive
if exist adaptive\*.la* del adaptive\*.la*
echo "Splitting input files into 256x256 tiles"
if "%2" == "" goto onedir
if "%3" == "" goto twodir
echo Merging 3 projects
echo Indexing input LAZ files
lasindex -i %1 -i %2 -i %3 -cores %CORES%
echo Tiling 3 projects
lastile -i %1 -i %2 -i %3 -cores %CORES% -flag_as_withheld -tile_size 256 -buffer 32 -full_bb -refine 1490000 -o adaptive\tile -olaz 2>NUL
goto tiled
:twodir
echo Merging 2 projects
echo Indexing input LAZ files
lasindex -i %1 -i %2 -cores %CORES%
echo Tiling 2 projects
lastile -i %1 -i %2 -cores %CORES% -flag_as_withheld -tile_size 256 -buffer 32 -full_bb -refine 1490000 -o adaptive\tile -olaz 2>NUL
goto tiled
:onedir
echo Indexing input LAZ files
lasindex -i %1 -cores %CORES%
echo Tiling single input dir
lastile -i %1 -cores %CORES% -flag_as_withheld -tile_size 256 -buffer 32 -full_bb -refine 1490000 -o adaptive\tile -olaz 2>NUL
:tiled
echo "Spitting too large tiles into 128x128 sub-tiles"
lastile -cores %CORES% -i adaptive\*.laz -refine_tiles 1490000 -olaz 2>NUL
echo "Spitting too large tiles into 64x64 sub-sub-tiles"
lastile -cores %CORES% -i adaptive\*.laz -refine_tiles 1490000 -olaz 2>NUL

:skip_tile
goto skip_ground
if "YES" == "%GROUND%" goto do_ground
if "yes" == "%GROUND%" goto do_ground
if "TRUE" == "%GROUND%" goto do_ground
if "true" == "%GROUND%" goto do_ground
if not "GROUND" == "%GROUND%" goto skip_ground

:do_ground
echo "Calculating extra ground points"
::lasground_new -cores %CORES% -i adaptive\*.laz -ilay -ignore_class 6 7 9 10 11 -hyper_fine -step 3.0 -compute_height -non_ground_unchanged -spike 3.0 -olay 2>NUL
lasground_new -cores %CORES% -i adaptive\*.laz -ignore_class 6 7 9 10 11 -fine -town -non_ground_unchanged -spike 3.0 -olay

:skip_ground
::pause
echo "Calculating point heights above ground TIN, saving in LAY layer"
lasheight -cores %CORES% -i adaptive\*.laz -ilay -ignore_class 7 9 10 11 -olay 2>NUL
echo "Classifying buildings, adding another LAY layer"
lasclassify -cores %CORES% -i adaptive\*.laz -ilay -ignore_class 7 9 10 11 -planar 0.1 -olay 2>NUL

md tiles_classified
if exist tiles_classified\*.la* del tiles_classified\*.la*
echo "Recalculating point heights and sorting into LOW/MEDIUM/HIGH vegetation"
lasheight -cores %CORES% -i adaptive\*.laz -ilay -ignore_class 6 9 10 11 -drop_class 7 -drop_below -5 -drop_above 100 -classify_between 0.3 1.3 3 -classify_between 1.3 3.5 4 -classify_between 3.5 100 5 -odir tiles_classified -olaz 2>NUL

:tiles_generated
echo Classified tiles generated, ready to generate contours, slope/cliff/dem and vegetation

:: Generate contours.dxf & contours.crt for ocad import. Picks output name from current directory
gen-contour-adaptive.pl -cores %CORES% -force -contour_intervals 25,5,1 tiles_classified\tile*.laz -skipzero -arg "-kill 50 -smooth 2"

:: Generate Digital Elevation Model in .ASC format as well as slope and cliff images
:: Merges the individual ASC tile files into a slope/cliff image and an ASC DEM
rem gen-slope-asc_thin.pl -cores %CORES% -force tiles_classified\tile*.laz
gen-slope-adaptive.pl -cores %CORES% -force tiles_classified\tile*.laz -cliff 50

:: Remove the temporary individual ASC files, one for each tile
del /q tiles_classified\tile*.asc

:: Generate vegetation model, uses the lasheight/lasclassify steps to determine vegetation points
gen-veg-adaptive.pl -cores %CORES% -force tiles_classified\tile*.laz

:: Convert vegetation to vector data
veg2dxf-adaptive.pl -cores %CORES% -merge tiles_classified\tile*.gif
:: Remove the temporary individual GIF/GFW files, one for each tile
del /q tiles_classified\tile*.gif
del /q tiles_classified\tile*.gfw
